﻿using System;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;
using System.IO.Ports;

namespace msxdisk
{
    public partial class SendROMForm : Form
    {
        const int blocksize = 2048;
        const int sleep = 100;
        public SerialPort rs232;
        public string romfile;

        public bool Reset = false;

        public SendROMForm()
        {
            InitializeComponent();
        }

        private void SendROMForm_Load(object sender, EventArgs e)
        {
            this.Text = "Sending " + Path.GetFileName(romfile) + "...";
        }

        private void SendROMForm_Shown(object sender, EventArgs e)
        {
            worker.RunWorkerAsync();
        }

        private void SendROMForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (worker.IsBusy)
                e.Cancel = true;
        }

        private void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                Stream fs = new FileStream(romfile, FileMode.Open, FileAccess.Read);
                long romsize = fs.Length;
                if (romsize > 32768) throw new Exception("Max ROM Size is 32KB");
                progress.Value = 0;
                byte[] len = new byte[] { (byte)(romsize & 255), (byte)(romsize >> 8) };
                Reset = true;
                rs232.Write(len, 0, 2);
                System.Threading.Thread.Sleep(sleep);
                byte[] data = new byte[blocksize];
                for (int i = 0; i < romsize; )
                {
                    fs.Read(data, 0, blocksize);
                    rs232.Write(data, 0, blocksize);
                    i += blocksize;
                    SetProgress((int)Math.Min((float)i  / (float)romsize * 1000F, 1000F));
                }
                fs.Close();
            }
            catch (Exception ex)
            {
                Error(ex.Message);
            }
        }

        private void worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.Close();
        }

        delegate void SetProgressCallback(int progress);

        private void SetProgress(int progress)
        {
            if (this.progress.InvokeRequired)
            {
                SetProgressCallback d = new SetProgressCallback(SetProgress);
                this.Invoke(d, new object[] { progress });
            }
            else
            {
                this.progress.Value = progress;
            }
        }

        delegate void ErrorCallback(string message);

        private void Error(string message)
        {
            if (this.InvokeRequired)
            {
                ErrorCallback d = new ErrorCallback(Error);
                this.Invoke(d, new object[] { message });
            }
            else
            {
                MessageBox.Show(this, message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
