﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Threading;

namespace msxdisk
{
    public partial class TapeDrive : UserControl
    {
        public TapeDrive()
        {
            InitializeComponent();
        }

        object lockCAS = new object();

        Cassette cas;
        byte[] cas_frame = new byte[Cassette.frameSize];

        int status = 0;

        public void LockCAS()
        {
            Monitor.Enter(lockCAS);
        }

        public void UnlockCAS()
        {
            Monitor.Exit(lockCAS);
        }

        public void SetStatus(int status)
        {
            this.status = status;
        }

        public void SetCAS(string casfile)
        {
            try
            {
                LockCAS();

                if (cas != null) cas.close();
                cas = null;

                pnlCAS.BackColor = Color.Black;
                txtCAS.Text = "";

                if (casfile != null)
                {
                    try
                    {
                        cas = new Cassette(casfile);
                    }
                    catch
                    {
                        MessageBox.Show(this, "Unable to open cassette " + Path.GetFileName(casfile), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }

                    pnlCAS.BackColor = Color.Blue;
                    txtCAS.Text = Path.GetFileName(casfile);
                }
            }
            finally
            {
                UnlockCAS();
            }
        }

        public bool TAPION()
        {
            try
            {
                LockCAS();

                if (cas != null)
                    return cas.tapion();
                else
                    return false;
            }
            finally
            {
                UnlockCAS();
            }
        }

        public bool TAPIN(byte[] frame)
        {
            try
            {
                LockCAS();

                if (cas != null)
                    return cas.tapin(frame);
                else
                    return false;
            }
            finally
            {
                UnlockCAS();
            }
        }

        public bool TAPIOF()
        {
            try
            {
                LockCAS();

                if (cas != null)
                    return cas.tapiof();
                else
                    return false;
            }
            finally
            {
                UnlockCAS();
            }
        }

        public bool TAPOON()
        {
            try
            {
                LockCAS();

                if (cas != null)
                    return cas.tapoon();
                else
                    return false;
            }
            finally
            {
                UnlockCAS();
            }
        }

        public bool TAPOUT_OK()
        {
            try
            {
                LockCAS();

                if (cas != null)
                    return cas.tapout_ok();
                else
                    return false;
            }
            finally
            {
                UnlockCAS();
            }
        }

        public bool TAPOUT(byte[] frame)
        {
            try
            {
                LockCAS();

                if (cas != null)
                    return cas.tapout(frame);
                else
                    return false;
            }
            finally
            {
                UnlockCAS();
            }
        }

        public bool TAPOOF()
        {
            try
            {
                LockCAS();

                if (cas != null)
                    return cas.tapoof();
                else
                    return false;
            }
            finally
            {
                UnlockCAS();
            }
        }

        delegate void SetPositionCallback(string position);

        public void SetPosition(string position)
        {
            if (this.txtPosition.InvokeRequired)
            {
                SetPositionCallback d = new SetPositionCallback(SetPosition);
                this.Invoke(d, new object[] { position });
            }
            else
            {
                txtPosition.Text = position;
            }
        }

        delegate void SetLEDColorCallback(Color color);

        void SetLEDColor(Color color)
        {
            if (this.panelLED.InvokeRequired)
            {
                SetLEDColorCallback d = new SetLEDColorCallback(SetLEDColor);
                this.Invoke(d, new object[] { color });
            }
            else
            {
                switch (status)
                {
                    case 0:
                        panelLED.BackColor = color;
                        break;
                    case -1:
                        panelLED.BackColor = color;
                        break;
                    case 1:
                        panelLED.BackColor = color;
                        break;
                }
            }
        }

        private void btnEject_Click(object sender, EventArgs e)
        {
            SetCAS(null);

            ofdCAS.FileName = "";
            if (ofdCAS.ShowDialog(this) == DialogResult.OK)
            {
                SetCAS(ofdCAS.FileName);
            }
        }

        private void btnRewind_Click(object sender, EventArgs e)
        {
            try
            {
                LockCAS();

                if (cas != null)
                    cas.rewind();
            }
            finally
            {
                UnlockCAS();
            }
        }

        private void tmrCAS_Tick(object sender, EventArgs e)
        {
            try
            {
                LockCAS();

                if (cas == null)
                {
                    SetPosition("0:00:00 / 0:00:00");
                }
                else
                {
                    int pos, len;
                    cas.get_pos(out pos, out len);
                    SetPosition(string.Format("{0:g} / {1:g}", TimeSpan.FromSeconds(pos / 109), TimeSpan.FromSeconds(len / 109)));
                }
            }
            finally
            {
                UnlockCAS();
            }

            SetLEDColor(status == 0 ? Color.DarkGreen :
                        status == 1 ? Color.LightGreen :
                        status == -1 ? Color.Pink : Color.DarkGreen);
        }
    }
}
